/// <reference types="node" />
import { EventEmitter } from "events";
import { NodeId } from "node-opcua-nodeid";
import { StatusCode } from "node-opcua-status-code";
import { Variant } from "node-opcua-variant";
import { ClientSession } from "../client_session";
export interface TVariant<T> extends Variant {
    value: T;
}
export interface TTwoStateStatus extends TVariant<string> {
    id: TVariant<boolean>;
}
export interface EventStuff {
    conditionId: TVariant<NodeId>;
    eventType: TVariant<NodeId>;
    eventId: TVariant<Buffer>;
    retain: TVariant<boolean>;
    activeState: TTwoStateStatus;
    ackedState: TTwoStateStatus;
    confirmedState: TTwoStateStatus;
}
export interface ClientAlarm {
    conditionId: NodeId;
    eventType: NodeId;
    fields: EventStuff;
    on(eventName: "changed", eventHandler: () => void): this;
    acknowledge(session: ClientSession, comment: string): Promise<StatusCode>;
}
/**
 * describes a OPCUA Alarm as seen in the client side
 */
export declare class ClientAlarm extends EventEmitter {
    conditionId: NodeId;
    eventType: NodeId;
    eventId: Buffer;
    fields: EventStuff;
    constructor(eventFields: EventStuff);
    confirm(session: ClientSession, comment: string): Promise<StatusCode>;
    update(eventFields: EventStuff): void;
    getRetain(): boolean;
}
export declare function fieldsToJson(fields: string[], eventFields: Variant[]): EventStuff;
