"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.promoteOpaqueStructure = void 0;
const node_opcua_extension_object_1 = require("node-opcua-extension-object");
const node_opcua_variant_1 = require("node-opcua-variant");
//
const get_extra_data_type_manager_1 = require("./get_extra_data_type_manager");
const resolve_dynamic_extension_object_1 = require("./resolve_dynamic_extension_object");
;
function promoteOpaqueStructure(session, dataValues) {
    return __awaiter(this, void 0, void 0, function* () {
        // count number of Opaque Structures
        const dataValuesToFix = dataValues.filter((dataValue) => dataValue.value && dataValue.value.dataType === node_opcua_variant_1.DataType.ExtensionObject &&
            ((dataValue.value.arrayType === node_opcua_variant_1.VariantArrayType.Scalar
                && dataValue.value.value instanceof node_opcua_extension_object_1.OpaqueStructure)
                ||
                    (dataValue.value.arrayType !== node_opcua_variant_1.VariantArrayType.Scalar
                        && dataValue.value.value && dataValue.value.value.length >= 0
                        && dataValue.value.value[0] instanceof node_opcua_extension_object_1.OpaqueStructure)));
        if (dataValuesToFix.length === 0) {
            return;
        }
        // construct dataTypeManager if not already present
        const extraDataTypeManager = yield (0, get_extra_data_type_manager_1.getExtraDataTypeManager)(session);
        const promises = dataValuesToFix.map((dataValue) => __awaiter(this, void 0, void 0, function* () {
            return yield (0, resolve_dynamic_extension_object_1.resolveDynamicExtensionObject)(session, dataValue.value, extraDataTypeManager);
        }));
        // https://medium.com/swlh/dealing-with-multiple-promises-in-javascript-41d6c21f20ff
        yield Promise.all(promises.map(p => p.catch(e => e)));
    });
}
exports.promoteOpaqueStructure = promoteOpaqueStructure;
//# sourceMappingURL=promote_opaque_structure.js.map